@@ -35,6 +35,7 @@ import org.apache.hc.client5.http.UserTokenHandler;
 import org.apache.hc.client5.http.auth.AuthSchemeFactory;
 import org.apache.hc.client5.http.auth.CredentialsProvider;
 import org.apache.hc.client5.http.config.TlsConfig;
+import org.apache.hc.client5.http.nio.AsyncClientConnectionManager;
 import org.apache.hc.core5.http.Header;
 import org.apache.hc.core5.http.HttpRequestInterceptor;
 import org.apache.hc.core5.http.HttpResponseInterceptor;
@@ -50,8 +51,12 @@ public interface TestAsyncClientBuilder {
 
     TestAsyncClientBuilder setTimeout(Timeout soTimeout);
 
-    default TestAsyncClientBuilder addResponseInterceptorFirst(final HttpResponseInterceptor interceptor) {
-        return this;
+    default TestAsyncClientBuilder setConnectionManager(AsyncClientConnectionManager connManager) {
+        throw new UnsupportedOperationException("Operation not supported by " + getProtocolLevel());
+    }
+
+    default TestAsyncClientBuilder addResponseInterceptorFirst(HttpResponseInterceptor interceptor) {
+        throw new UnsupportedOperationException("Operation not supported by " + getProtocolLevel());
     }
 
     default TestAsyncClientBuilder addResponseInterceptorLast(HttpResponseInterceptor interceptor) {
